@@ -11,11 +11,12 @@
 #############################################################################
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
+$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
 umask(0002); # make sure the openwebmail group can write
 
@@ -47,13 +48,16 @@ $firstmessage = param("firstmessage") || 1;
 $sort = param("sort") || $prefs{'sort'} || 'date';
 
 # extern vars
-use vars qw($lang_charset %lang_folders %lang_text %lang_err);	# defined in lang/xy
+use vars qw(%lang_folders %lang_text %lang_err);	# defined in lang/xy
+use vars qw($_OFFSET $_STATUS);				# defined in maildb.pl
 
 ########################## MAIN ##############################
 
 my $action = param("action");
 if ($action eq "editfolders") {
    editfolders();
+} elsif ($action eq "markreadfolder") {
+   markreadfolder();
 } elsif ($action eq "chkindexfolder") {
    reindexfolder(0);
 } elsif ($action eq "reindexfolder") {
@@ -81,10 +85,10 @@ sub editfolders {
    $total_allmessages=0;
    $total_foldersize=0;
 
-   push(@defaultfolders, 'INBOX', 
-                         'saved-messages', 
-                         'sent-mail', 
-                         'saved-drafts', 
+   push(@defaultfolders, 'INBOX',
+                         'saved-messages',
+                         'sent-mail',
+                         'saved-drafts',
                          'mail-trash');
 
    foreach (@validfolders) {
@@ -138,7 +142,7 @@ sub editfolders {
 
    $temphtml = textfield(-name=>'foldername',
                          -default=>'',
-                         -size=> $config{'foldername_maxlen'},
+                         -size=> 24,
                          -maxlength=>$config{'foldername_maxlen'},
                          -override=>'1');
 
@@ -190,16 +194,13 @@ sub editfolders {
       $usagestr="&nbsp;";
    }
 
-   if ($total_foldersize > 1048575){
-      $total_foldersize = int($total_foldersize/1048576*10+0.5)/10 . "MB";
-   } elsif ($total_foldersize > 1023) {
-      $total_foldersize =  int(($total_foldersize/1024)+0.5) . "KB";
-   }
+   $total_foldersize=lenstr($total_foldersize,0);
+
    $temphtml = qq|<tr>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$lang_text{'total'}</B></td>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$total_newmessages</B></td>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>$total_allmessages</B></td>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>$total_foldersize</B></td>|.
+               qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_allmessages</B></td>|.
+               qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_foldersize</B></td>|.
                qq|<td bgcolor=$bgcolor align="center">$usagestr</td>|.
                qq|</tr>|;
    $html =~ s/\@\@\@TOTAL\@\@\@/$temphtml/;
@@ -217,8 +218,8 @@ sub _folderline {
    my (%HDB, $newmessages, $allmessages, $foldersize);
    my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
 
-   if ( -f "$headerdb$config{'dbm_ext'}" ) {
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
+   if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
       dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       if ( defined($HDB{'ALLMESSAGES'}) ) {
          $allmessages=$HDB{'ALLMESSAGES'};
@@ -233,7 +234,7 @@ sub _folderline {
          $newmessages='&nbsp;';
       }
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
    } else {
       $allmessages='&nbsp;';
       $newmessages='&nbsp;';
@@ -243,12 +244,7 @@ sub _folderline {
    $foldersize = (-s "$folderfile") + (-s "$headerdb$config{'dbm_ext'}");
 
    $total_foldersize+=$foldersize;
-   # round foldersize and change to an appropriate unit for display
-   if ($foldersize > 1048575){
-      $foldersize = int($foldersize/1048576*10+0.5)/10 . "MB";
-   } elsif ($foldersize > 1023) {
-      $foldersize =  int(($foldersize/1024)+0.5) . "KB";
-   }
+   $foldersize=lenstr($foldersize,0);
 
    my $escapedcurrfolder = escapeURL($currfolder);
    my $url = "$config{'ow_cgiurl'}/openwebmail-folder.pl?sessionid=$thissession&amp;folder=$escapedcurrfolder&amp;action=downloadfolder";
@@ -262,8 +258,8 @@ sub _folderline {
                 qq|<a href="$url" title="$lang_text{'download'} $folderstr"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/download.gif" align="absmiddle" border="0" ALT="$lang_text{'download'} $folderstr">|.
                 qq|</a></td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
-                qq|<td align="center" bgcolor=$bgcolor>$allmessages</td>|.
-                qq|<td align="center" bgcolor=$bgcolor>$foldersize</td>\n|;
+                qq|<td align="center" bgcolor=$bgcolor>&nbsp;$allmessages</td>|.
+                qq|<td align="center" bgcolor=$bgcolor>&nbsp;$foldersize</td>\n|;
 
    $temphtml .= qq|<td bgcolor=$bgcolor align="center">\n|;
 
@@ -292,15 +288,19 @@ sub _folderline {
                        -override=>'1');
    $temphtml .= "\n";
 
-   my $jsfolderstr=$currfolder; $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
+   my $jsfolderstr=$lang_folders{$currfolder}||$currfolder; 
+   $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
+   $temphtml .= submit(-name=>"$lang_text{'markread'}",
+                       -class=>"medtext",
+                       -onClick=>"return OpConfirm('folderform$i', 'markreadfolder', $lang_text{'foldermarkreadconf'}+' ( $jsfolderstr )')");
    $temphtml .= submit(-name=>"$lang_text{'chkindex'}",
                        -class=>"medtext",
                        -onClick=>"return OpConfirm('folderform$i', 'chkindexfolder', $lang_text{'folderchkindexconf'}+' ( $jsfolderstr )')");
-   $temphtml .= submit(-name=>"$lang_text{'reindex'}", 
+   $temphtml .= submit(-name=>"$lang_text{'reindex'}",
                        -class=>"medtext",
                        -onClick=>"return OpConfirm('folderform$i', 'reindexfolder', $lang_text{'folderreindexconf'}+' ( $jsfolderstr )')");
    if ($currfolder ne "INBOX") {
-      $temphtml .= submit(-name=>"$lang_text{'rename'}", 
+      $temphtml .= submit(-name=>"$lang_text{'rename'}",
                           -class=>"medtext",
                           -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $jsfolderstr )')");
       $temphtml .= submit(-name=>"$lang_text{'delete'}",
@@ -315,12 +315,87 @@ sub _folderline {
 }
 ################### END EDITFOLDERS ########################
 
+################### MARKREADFOLDER ##############################
+sub markreadfolder {
+   my $foldertomark = param('foldername') || '';
+   $foldertomark =~ s!\.\.+/!!g;
+   $foldertomark =~ s!^\s*/!!g;
+   $foldertomark =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
+   ($foldertomark =~ /^(.+)$/) && ($foldertomark = $1);
+   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertomark);
+
+   filelock($folderfile, LOCK_EX|LOCK_NB) or openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
+
+   update_headerdb($headerdb, $folderfile);
+
+   my (%HDB, %offset, %status);
+   filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
+   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+   foreach my $messageid (keys %HDB) {
+      next if ( $messageid eq 'METAINFO'
+             || $messageid eq 'NEWMESSAGES'
+             || $messageid eq 'INTERNALMESSAGES'
+             || $messageid eq 'ALLMESSAGES'
+             || $messageid eq "" );
+      my @attr=split( /@@@/, $HDB{$messageid} );
+      if ($attr[$_STATUS] !~ /R/i) {
+         $offset{$messageid}=$attr[$_OFFSET];
+         $status{$messageid}=$attr[$_STATUS];
+      }
+   }
+   dbmclose(%HDB);
+   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+
+   my @markids;
+   my $tmpfile="/tmp/markread_tmp_$$";
+   my $tmpdb="/tmp/.markread_tmp_$$";
+   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);
+   ($tmpdb =~ /^(.+)$/) && ($tmpdb = $1);
+
+   filelock("$tmpfile", LOCK_EX);
+
+   update_headerdb($tmpdb, $tmpfile);
+
+   foreach my $messageid (sort { $offset{$a}<=>$offset{$b} } keys %offset) {
+      my @copyid;
+      push(@copyid, $messageid);
+      if (operate_message_with_ids("copy", \@copyid,
+   					$folderfile, $headerdb, $tmpfile, $tmpdb) >0 ) {
+         update_message_status($messageid, $status{$messageid}."R", $tmpdb, $tmpfile);
+         push(@markids, $messageid);
+
+         if ($#markids>=99) { # flush per 100 msgs
+            operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+            operate_message_with_ids("move", \@markids,
+   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+            @markids=();
+         }
+
+      }
+   }
+   operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+   operate_message_with_ids("move", \@markids,
+   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+      
+   filelock("$tmpfile", LOCK_UN);
+   filelock($folderfile, LOCK_UN);
+   unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
+
+   writelog("markread folder - $foldertomark");
+   writehistory("markread folder - $foldertomark");
+
+   getfolders(\@validfolders, \$folderusage);
+   editfolders();
+}
+################### END MARKREADFOLDER ##########################
+
 ################### REINDEXFOLDER ##############################
 sub reindexfolder {
    my $recreate=$_[0];
    my $foldertoindex = param('foldername') || '';
-   $foldertoindex =~ s/\.\.+//g;
-   $foldertoindex =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertoindex =~ s!\.\.+/!!g;
+   $foldertoindex =~ s!^\s*/!!g;
+   $foldertoindex =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertoindex =~ /^(.+)$/) && ($foldertoindex = $1);
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoindex);
 
@@ -332,26 +407,34 @@ sub reindexfolder {
 
    if ( -f "$headerdb$config{'dbm_ext'}" ) {
       my %HDB;
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
       dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       $HDB{'METAINFO'}={'RENEW'};
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
    }
    update_headerdb($headerdb, $folderfile);
 
-   getfolders(\@validfolders, \$folderusage);
-   editfolders();
+   if ($recreate) {
+      writelog("reindex folder - $foldertoindex");
+      writehistory("reindex folder - $foldertoindex");
+   } else {
+      writelog("chkindex folder - $foldertoindex");
+      writehistory("chkindex folder - $foldertoindex");
+   }
 
 #   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&firstmessage=$firstmessage\n\n";
+   getfolders(\@validfolders, \$folderusage);
+   editfolders();
 }
 ################### END REINDEXFOLDER ##########################
 
 ################### ADDFOLDER ##############################
 sub addfolder {
    my $foldertoadd = param('foldername') || '';
-   $foldertoadd =~ s/\.\.+//g;
-   $foldertoadd =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertoadd =~ s!\.\.+/!!g;
+   $foldertoadd =~ s!^\s*/!!g;
+   $foldertoadd =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertoadd =~ /^(.+)$/) && ($foldertoadd = $1);
 
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
@@ -386,7 +469,7 @@ sub addfolder {
 sub is_defaultfolder {
    my $foldername=$_[0];
    if ($foldername eq 'INBOX' ||
-       $foldername eq 'saved-messages' || 
+       $foldername eq 'saved-messages' ||
        $foldername eq 'sent-mail' ||
        $foldername eq 'saved-drafts' ||
        $foldername eq 'mail-trash' ||
@@ -405,8 +488,9 @@ sub is_defaultfolder {
 ################### DELETEFOLDER ##############################
 sub deletefolder {
    my $foldertodel = param('foldername') || '';
-   $foldertodel =~ s/\.\.+//g;
-   $foldertodel =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertodel =~ s!\.\.+/!!g;
+   $foldertodel =~ s!^\s*/!!g;
+   $foldertodel =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertodel =~ /^(.+)$/) && ($foldertodel = $1);
 
    # if is INBOX, return to editfolder immediately
@@ -416,14 +500,16 @@ sub deletefolder {
    }
 
    if ( -f "$folderdir/$foldertodel" ) {
+      my $headerdb="$folderdir/$foldertodel";
+      ($headerdb =~ /^(.+)\/(.*)$/) && ($headerdb = "$1/.$2");
       unlink ("$folderdir/$foldertodel",
-              "$folderdir/.$foldertodel$config{'dbm_ext'}",
-              "$folderdir/.$foldertodel.db",
-	      "$folderdir/.$foldertodel.dir",
-              "$folderdir/.$foldertodel.pag",
-              "$folderdir/.$foldertodel.cache",
               "$folderdir/$foldertodel.lock",
-              "$folderdir/$foldertodel.lock.lock");              
+              "$folderdir/$foldertodel.lock.lock",
+              "$headerdb$config{'dbm_ext'}",
+              "$headerdb.db",
+	      "$headerdb.dir",
+              "$headerdb.pag",
+              "$headerdb.cache");
 
       writelog("delete folder - $foldertodel");
       writehistory("delete folder - $foldertodel");
@@ -437,8 +523,9 @@ sub deletefolder {
 ################### RENAMEFOLDER ##############################
 sub renamefolder {
    my $oldname = param('foldername') || '';
-   $oldname =~ s/\.\.+//g;
-   $oldname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $oldname =~ s!\.\.+/!!g;
+   $oldname =~ s!^\s*/!!g;
+   $oldname =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($oldname =~ /^(.+)$/) && ($oldname = $1);
 
    if ($oldname eq 'INBOX') {
@@ -447,8 +534,9 @@ sub renamefolder {
    }
 
    my $newname = param('foldernewname');
-   $newname =~ s/\.\.+//g;
-   $newname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $newname =~ s!\.\.+/!!g;
+   $newname =~ s!^\s*/!!g;
+   $newname =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($newname =~ /^(.+)$/) && ($newname = $1);
 
    if (length($newname) > $config{'foldername_maxlen'}) {
@@ -463,13 +551,17 @@ sub renamefolder {
    }
 
    if ( -f "$folderdir/$oldname" ) {
-      rename("$folderdir/$oldname",          "$folderdir/$newname");
-      rename("$folderdir/.$oldname$config{'dbm_ext'}", "$folderdir/.$newname$config{'dbm_ext'}");
-      rename("$folderdir/.$oldname.db",      "$folderdir/.$newname.db");
-      rename("$folderdir/.$oldname.dir",     "$folderdir/.$newname.dir");
-      rename("$folderdir/.$oldname.pag",     "$folderdir/.$newname.pag");
-      rename("$folderdir/.$oldname.cache",   "$folderdir/.$newname.cache");
-      unlink("$folderdir/$oldname.lock", "$folderdir/$oldname.lock.lock");
+      my $oldheaderdb="$folderdir/$oldname";
+      my $newheaderdb="$folderdir/$newname";
+      ($oldheaderdb =~ /^(.+)\/(.*)$/) && ($oldheaderdb = "$1/.$2");
+      ($newheaderdb =~ /^(.+)\/(.*)$/) && ($newheaderdb = "$1/.$2");
+      rename("$folderdir/$oldname",            "$folderdir/$newname");
+      rename("$oldheaderdb$config{'dbm_ext'}", "$newheaderdb$config{'dbm_ext'}");
+      rename("$oldheaderdb.db",                "$newheaderdb.db");
+      rename("$oldheaderdb.dir",               "$newheaderdb.dir");
+      rename("$oldheaderdb.pag",               "$newheaderdb.pag");
+      rename("$oldheaderdb.cache",             "$newheaderdb.cache");
+      unlink("$folderdir/$newname.lock",       "$folderdir/$oldname.lock.lock");
 
       writelog("rename folder - rename $oldname to $newname");
       writehistory("rename folder - rename $oldname to $newname");
