@@ -3,7 +3,7 @@
 # Open WebMail - Provides a web interface to user mailboxes                 #
 #                                                                           #
 # Copyright (C) 2001-2002                                                   #
-# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric                 #
+# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung   #
 # Copyright (C) 2000                                                        #
 # Ernie Miller  (original GPL project: Neomail)                             #
 #                                                                           #
@@ -11,18 +11,19 @@
 #############################################################################
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
+$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
-umask(0007); # make sure the openwebmail group can write
+umask(0002); # make sure the openwebmail group can write
 
 use strict;
 use Fcntl qw(:DEFAULT :flock);
 use Socket;
-use CGI qw(:standard);
+use CGI qw(-private_tempfiles :standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
@@ -36,7 +37,7 @@ use vars qw(%prefs %style);
 use vars qw($folderdir);
 
 # extern vars
-use vars qw($lang_charset %lang_text %lang_err);	# defined in lang/xy
+use vars qw(%lang_text %lang_err);	# defined in lang/xy
 use vars qw($pop3_authserver);	# defined in auth_pop3.pl
 
 readconf(\%config, \%config_raw, "$SCRIPT_DIR/etc/openwebmail.conf.default");
@@ -48,34 +49,35 @@ if ( $>!=0 && ($config{'use_homedirspools'}||$config{'use_homedirfolders'}) ) {
 }
 
 # validate allowed_serverdomain
-my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
+my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
 if (! is_serverdomain_allowed($httphost) ) {
    print "Content-type: text/html\n\nService is not available for domain  ' $httphost '";
    exit 0;
 }
 
-if ( $config{'logfile'} ne 'no' && ! -f $config{'logfile'} ) {
+if ( ($config{'logfile'} ne 'no') ) {
    my $mailgid=getgrnam('mail');
-   open (LOGFILE,">>$config{'logfile'}") or 
-      openwebmailerror("Can't open log file $config{'logfile'}!");
-   close(LOGFILE);
-   chmod(0660, $config{'logfile'});
-   chown($>, $mailgid, $config{'logfile'});
+   my ($fmode, $fuid, $fgid) = (stat($config{'logfile'}))[2,4,5];
+   if ( !($fmode & 0100000) ) {
+      open (LOGFILE,">>$config{'logfile'}") or
+         openwebmailerror("Can't open log file $config{'logfile'}!");
+      close(LOGFILE);
+   }
+   chmod(0660, $config{'logfile'}) if (($fmode&0660)!=0660);
+   chown($>, $mailgid, $config{'logfile'}) if ($fuid!=$>||$fgid!=$mailgid);
 }
 
-%prefs = %{&readprefs};
-%style = %{&readstyle};
-
-($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
-require "etc/lang/$prefs{'language'}";
-$lang_charset ||= 'iso-8859-1';
-
 ####################### MAIN ##########################
+
 if ( param("loginname") && param("password") ) {
    $loginname=param("loginname");
-   if ($loginname!~/\@/ && param("logindomain") ne "") {
-      $loginname .= "\@".param("logindomain");
+   $loginname.='@'.param("logindomain") if ($loginname!~/\@/ && param("logindomain") ne "");
+   if ($config{'case_insensitive_login'}) {
+      $loginname=lc($loginname);
+   } else {
+      $loginname=$1.'@'.lc($2) if ($loginname=~/^(.+)\@(.+)$/);
    }
+
    my $siteconf;
    if ($loginname=~/\@(.+)$/) {
        my $domain=$1;
@@ -84,29 +86,52 @@ if ( param("loginname") && param("password") ) {
        }
        $siteconf="$config{'ow_etcdir'}/sites.conf/$domain";
    } else {
-       my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
+       my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
        $siteconf="$config{'ow_etcdir'}/sites.conf/$httphost";
    }
-   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf"); 
+   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf");
 
    require $config{'auth_module'} or
       openwebmailerror("Can't open authentication module $config{'auth_module'}");
 
-   if ( ($config{'logfile'} ne 'no') && (! -f $config{'logfile'})  ) {
+   if ( ($config{'logfile'} ne 'no') ) {
       my $mailgid=getgrnam('mail');
-      open (LOGFILE,">>$config{'logfile'}") or 
-         openwebmailerror("Can't open log file $config{'logfile'}!");
-      close(LOGFILE);
-      chmod(0660, $config{'logfile'});
-      chown($>, $mailgid, $config{'logfile'});
+      my ($fmode, $fuid, $fgid) = (stat($config{'logfile'}))[2,4,5];
+      if ( !($fmode & 0100000) ) {
+         open (LOGFILE,">>$config{'logfile'}") or
+            openwebmailerror("Can't open log file $config{'logfile'}!");
+         close(LOGFILE);
+      }
+      chmod(0660, $config{'logfile'}) if (($fmode&0660)!=0660);
+      chown($>, $mailgid, $config{'logfile'}) if ($fuid!=$>||$fgid!=$mailgid);
    }
 
-   my $virtname=$config{'virtusertable'}; 
+   my $virtname=$config{'virtusertable'};
    $virtname=~s!/!.!g; $virtname=~s/^\.+//;
    update_virtusertable("$config{'ow_etcdir'}/$virtname", $config{'virtusertable'});
 
+   # check & create mapping table for chinese BIG5/GB, solar/lunar convertion
+   mkdb_b2g();
+   mkdb_g2b();
+   mkdb_lunar();
+
+   %prefs = %{&readprefs};
+   %style = %{&readstyle};
+   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
+   require "etc/lang/$prefs{'language'}";
+
    login();
+
 } else {            # no action has been taken, display login page
+   my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
+   my $siteconf="$config{'ow_etcdir'}/sites.conf/$httphost";
+   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf");
+
+   %prefs = %{&readprefs};
+   %style = %{&readstyle};
+   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
+   require "etc/lang/$prefs{'language'}";
+
    loginmenu();
 }
 
@@ -129,22 +154,42 @@ sub loginmenu {
 
    $temphtml = startform(-action=>"$config{'ow_cgiurl'}/openwebmail.pl",
                          -name=>'login');
-   $temphtml .= hidden("action","login");
+   if (defined(param('action'))) {
+      $temphtml .= hidden("action", param('action'));
+      $temphtml .= hidden("to", param('to')) if (defined(param('to')));
+      $temphtml .= hidden("subject", param('subject')) if (defined(param('subject')));
+   }
    $html =~ s/\@\@\@STARTFORM\@\@\@/$temphtml/;
-   $temphtml = textfield(-name=>'loginname',
-                         -default=>'',
-                         -size=>'12',
-                         -onChange=>'focuspwd()', 
-                         -override=>'1');
+
+   if (cookie("openwebmail-rememberme")) {
+      $temphtml = textfield(-name=>'loginname',
+                            -default=>cookie("openwebmail-loginname"),
+                            -size=>'12',
+                            -onChange=>'focuspwd()',
+                            -override=>'1');
+   } else {
+      $temphtml = textfield(-name=>'loginname',
+                            -default=>'' ,
+                            -size=>'12',
+                            -onChange=>'focuspwd()',
+                            -override=>'1');
+   }
    $html =~ s/\@\@\@USERIDFIELD\@\@\@/$temphtml/;
+
    $temphtml = password_field(-name=>'password',
                               -default=>'',
                               -size=>'12',
-                              -onChange=>'focuslogin()', 
+                              -onChange=>'focuslogin()',
                               -override=>'1');
    $html =~ s/\@\@\@PASSWORDFIELD\@\@\@/$temphtml/;
 
-   if ( $#{$config{'domainnames'}} >0 ) {
+   $temphtml = checkbox(-name=>'rememberme',
+                        -value=>'1',
+                        -checked=>cookie("openwebmail-rememberme")||0,
+                        -label=>'');
+   $html =~ s/\@\@\@REMEMBERMECHECKBOX\@\@\@/$temphtml/;
+
+   if ( $#{$config{'domainnames'}} >0 && $config{'enable_domainselectmenu'} ) {
       $temphtml = popup_menu(-name=>'logindomain',
                              -values=>[@{$config{'domainnames'}}] );
       $html =~ s/\@\@\@DOMAINMENU\@\@\@/$temphtml/;
@@ -177,7 +222,13 @@ sub login {
    $password = $1;
 
    ($loginname, $domain, $user, $userrealname, $uuid, $ugid, $homedir)
-	=get_domain_user_userinfo($loginname);
+					=get_domain_user_userinfo($loginname);
+
+   ($user =~ /^(.+)$/) && ($user = $1);		# untaint...
+   ($uuid =~ /^(.+)$/) && ($uuid = $1);
+   ($ugid =~ /^(.+)$/) && ($ugid = $1);
+   ($homedir =~ /^(.+)$/) && ($homedir = $1);
+
    if ($user eq "") {
       sleep $config{'loginerrordelay'};	# delayed response
       openwebmailerror("$lang_err{'user_not_exist'}");
@@ -199,7 +250,7 @@ sub login {
    if ( $errorcode==0 ) {
       my $userconf="$config{'ow_etcdir'}/users.conf/$user";
       $userconf .= "\@$domain" if ($config{'auth_withdomain'});
-      readconf(\%config, \%config_raw, "$userconf") if ( -f "$userconf"); 
+      readconf(\%config, \%config_raw, "$userconf") if ( -f "$userconf");
 
       my $clientip=get_clientip();
       # validate client ip
@@ -229,7 +280,7 @@ sub login {
             $clientdomain=ip2hostname($clientip) if ($clientdomain eq "");
             if ($clientdomain=~/\Q$token\E$/ || # matched
                 $clientdomain!~/\./) { 		# shortname in /etc/hosts
-               $allowed=1; last; 
+               $allowed=1; last;
             }
          }
          if (!$allowed) {
@@ -237,12 +288,23 @@ sub login {
          }
       }
 
-      # search old alive session and deletes old expired sessionids 
-      $thissession = search_and_cleanoldsessions(cookie("$user-sessionid")); 
+      # search old alive session and deletes old expired sessionids
+      $thissession = search_and_cleanoldsessions(cookie("$user-sessionid"));
       if ($thissession eq "") {
          $thissession = $loginname. "-session-" . rand(); # name the sessionid
       }
       writelog("login - $thissession");
+      ($thissession =~ /^(.+)$/) && ($thissession = $1);  # untaint
+
+      # create the user's home directory if necessary.
+      # this must be done before changing to the user's uid.
+      if ( $config{'create_homedir'} && ! -d "$homedir" ) {
+         if (mkdir ("$homedir", oct(700)) and chown($uuid, $ugid, $homedir)) {
+            writelog("mkdir - $homedir, uid=$uuid, gid=$ugid");
+         } else {
+            openwebmailerror("$lang_err{'cant_create_dir'} $homedir");
+         }
+      }
 
       if ( $config{'use_homedirspools'} || $config{'use_homedirfolders'} ) {
          my $mailgid=getgrnam('mail');
@@ -258,18 +320,15 @@ sub login {
          $folderdir = "$config{'ow_etcdir'}/users/$user";
          $folderdir .= "\@$domain" if ($config{'auth_withdomain'});
       }
-
-      ($thissession =~ /^(.+)$/) && ($thissession = $1);  # untaint ...
-      ($user =~ /^(.+)$/) && ($user = $1);
-      ($uuid =~ /^(.+)$/) && ($uuid = $1);
-      ($ugid =~ /^(.+)$/) && ($ugid = $1);
-      ($homedir =~ /^(.+)$/) && ($homedir = $1);
-      ($folderdir =~ /^(.+)$/) && ($folderdir = $1);
+      ($folderdir =~ /^(.+)$/) && ($folderdir = $1);	# untaint
 
       # create folderdir if it doesn't exist
       if (! -d "$folderdir" ) {
-         mkdir ("$folderdir", oct(700)) or
+         if (mkdir ("$folderdir", oct(700))) {
+            writelog("mkdir - $folderdir, euid=$>, egid=$)");
+         } else {
             openwebmailerror("$lang_err{'cant_create_dir'} $folderdir");
+         }
       }
 
       # create system spool file /var/mail/xxxx
@@ -281,20 +340,20 @@ sub login {
       }
 
       # create session file
-      my $setcookie;
+      my $sessioncookie_value;
       if ( -f "$config{'ow_etcdir'}/sessions/$thissession" ) { # continue an old session?
-         $setcookie = cookie("$user-sessionid");
+         $sessioncookie_value = cookie("$user-sessionid");
       } else {						       # a brand new sesion?
-         $setcookie = crypt(rand(),'OW');
+         $sessioncookie_value = crypt(rand(),'OW');
       }
       open (SESSION, "> $config{'ow_etcdir'}/sessions/$thissession") or # create sessionid
          openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions/$thissession!");
-      print SESSION $setcookie, "\n";
+      print SESSION $sessioncookie_value, "\n";
       print SESSION get_clientip(), "\n";
       close (SESSION);
       writehistory("login - $thissession");
 
-      # symbolic link ~/mbox to ~/mail/saved-messages 
+      # symbolic link ~/mbox to ~/mail/saved-messages
       if ( $config{'symboliclink_mbox'} &&
            ((lstat("$homedir/mbox"))[2] & 07770000) eq 0100000) { # regular file
          if (filelock("$folderdir/saved-messages", LOCK_EX|LOCK_NB)) {
@@ -349,7 +408,7 @@ sub login {
             }
 
             if ($pop3host ne $pop3_authserver ||
-                $pop3user ne $login || 
+                $pop3user ne $login ||
                 $pop3pass ne $password) {
                if ($pop3host ne $pop3_authserver || $pop3user ne $login) {
                   $pop3lastid="none";
@@ -371,25 +430,42 @@ sub login {
          $url="$config{'ow_cgiurl'}/openwebmail-prefs.pl?sessionid=$thissession&action=firsttimeuser";
       } elsif ( $rc_upgrade ) {
          $url="$config{'ow_cgiurl'}/openwebmail-prefs.pl?sessionid=$thissession&action=editprefs";
-      } elsif ( defined(param("to")) ) {
+      } elsif ( param('action') eq 'composemessage' ) {
          my $to=param("to");
          my $subject=param("subject");
          $url="$config{'ow_cgiurl'}/openwebmail-send.pl?sessionid=$thissession&action=composemessage&to=$to&subject=$subject";
+      } elsif ( param('action') eq 'calyear' || param('action') eq 'calmonth' ||
+                param('action') eq 'calweek' || param('action') eq 'calday' ) {
+         my $action=param('action');
+         $url="$config{'ow_cgiurl'}/openwebmail-cal.pl?sessionid=$thissession&action=$action";
       } else {
          $url="$config{'ow_cgiurl'}/openwebmail-main.pl?sessionid=$thissession&action=displayheaders_afterlogin";
       }
 
       my @headers=();
       push(@headers, -pragma=>'no-cache');
-      my $cookie = cookie( -name    => "$user-sessionid",
-                           -value   => "$setcookie",
-                           -path    => '/' );
-      push(@headers, -cookie=>$cookie);
-      push(@headers, -charset=>$lang_charset) if ($CGI::VERSION>=2.57);
+
+      # cookie for openwebmail to verify session, expired if browser closed
+      my $cookie1 = cookie( -name  => "$user-sessionid",
+                            -value => "$sessioncookie_value",
+                            -path  => '/');
+      # cookie for other ap to find openwebmail loginname, expired until 1 month later
+      my $cookie2 = cookie( -name  => "openwebmail-loginname",
+                            -value => "$loginname",
+                            -path  => '/',
+                            -expires => "+1M" );
+      # cookie for remember loginname switch, expired until 1 month later
+      my $cookie3 = cookie( -name  => "openwebmail-rememberme",
+                            -value => param("rememberme")||'',
+                            -path  => '/',
+                            -expires => "+1M" );
+
+      push(@headers, -cookie=>[$cookie1, $cookie2, $cookie3]);
+      push(@headers, -charset=>$prefs{'charset'}) if ($CGI::VERSION>=2.57);
 
       # load page with Refresh header only if not MSIE on Mac
       my $refresh;
-      if ($ENV{'HTTP_USER_AGENT'} !~ /MSIE.+Mac/) { 
+      if ($ENV{'HTTP_USER_AGENT'} !~ /MSIE.+Mac/) {
          # push(@headers, -Refresh=>"0;URL=$url");
          $refresh=qq|<meta http-equiv="refresh" content="0;URL=$url">|;
       }
@@ -428,7 +504,7 @@ sub login {
 		qq|</font></a>\n\n|;
 
       # load page with java script if MSIE on Mac
-      if ($ENV{'HTTP_USER_AGENT'}=~/MSIE.+Mac/) { 
+      if ($ENV{'HTTP_USER_AGENT'}=~/MSIE.+Mac/) {
          print  qq|<script language="JavaScript">\n<!--\n|,
 		qq|setTimeout("window.open('$url','_self')", 1*1000);\n|,
 		qq|//-->\n</script>\n|;
@@ -487,7 +563,7 @@ sub login {
       $html =~ s/\@\@\@ERRORMSG\@\@\@/$errormsg/;
 
       $html = applystyle($html);
-      
+
       print $html;
       printfooter(1);
       exit 0;
@@ -526,14 +602,15 @@ sub is_serverdomain_allowed {
 }
 ############### END IS_SERVERDOMAIN_ALLOWED ###################
 
-################ CLEANUPOLDSESSIONS ##################
-# delete expired session files and 
+################ SEARCH_AND_CLEANOLDSESSIONS ##################
+# delete expired session files and
 # try to find old session that is still valid for the same user cookie
 sub search_and_cleanoldsessions {
    my $oldcookie=$_[0];
    my $oldsessionid="";
-
    my $sessionid;
+   my @delfiles;
+
    opendir (SESSIONSDIR, "$config{'ow_etcdir'}/sessions") or
       openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions!");
    while (defined($sessionid = readdir(SESSIONSDIR))) {
@@ -542,8 +619,8 @@ sub search_and_cleanoldsessions {
          if ($sessionid =~ /^$loginname\-session\-0./) { # remove user old session if timeout
             if ( -M "$config{'ow_etcdir'}/sessions/$sessionid" > $prefs{'sessiontimeout'}/60/24 ) {
                writelog("session cleanup - $sessionid");
-               unlink "$config{'ow_etcdir'}/sessions/$sessionid";
-            } else {	# remove user old session from same client 
+               push(@delfiles, "$config{'ow_etcdir'}/sessions/$sessionid");
+            } else {	# remove user old session from same client
                open (SESSION, "$config{'ow_etcdir'}/sessions/$sessionid");
                my $cookie = <SESSION>; chomp $cookie;
                my $ip = <SESSION>; chomp $ip;
@@ -553,22 +630,23 @@ sub search_and_cleanoldsessions {
                      $oldsessionid=$sessionid;
                   } else {
                      writelog("session cleanup - $sessionid");
-                     unlink "$config{'ow_etcdir'}/sessions/$sessionid";
+                     push(@delfiles, "$config{'ow_etcdir'}/sessions/$sessionid");
                   }
                }
             }
          } else {	# remove others old session if more than 1 day
             if ( -M "$config{'ow_etcdir'}/sessions/$sessionid" > 1 ) {
                writelog("session cleanup - $sessionid");
-               unlink "$config{'ow_etcdir'}/sessions/$sessionid";
+               push(@delfiles, "$config{'ow_etcdir'}/sessions/$sessionid");
             }
          }
       }
    }
    closedir (SESSIONSDIR);
+   unlink(@delfiles) if ($#delfiles>=0);
    return($oldsessionid);
 }
-############## END CLEANUPOLDSESSIONS ################
+############## END SEARCH_AND_CLEANOLDSESSIONS ################
 
 #################### RELEASEUPGRADE ####################
 # convert file format from old release for backward compatibility
@@ -677,7 +755,7 @@ sub releaseupgrade {
          writehistory("release upgrade - $folderdir/*.cache by 20011216");
          writelog("release upgrade - $folderdir/*.cache by 20011216");
          # remove old .cache since its format is not compatible with new one
-         unlink(@cachefiles); 
+         unlink(@cachefiles);
       }
    }
 
@@ -688,28 +766,27 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
-
-         next if ( ! -f "$headerdb$config{'dbm_ext'}");
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
 
          filelock($folderfile, LOCK_SH);
          open (FOLDER, $folderfile);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
          dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
 
          if ( $HDB{'METAINFO'} eq metainfo($folderfile) ) { # upgrade only if hdb is uptodate
             @messageids=keys %HDB;
             foreach my $id (@messageids) {
-               next if ( $id eq 'METAINFO' 
-                || $id eq 'NEWMESSAGES' 
-                || $id eq 'INTERNALMESSAGES' 
-                || $id eq 'ALLMESSAGES' 
+               next if ( $id eq 'METAINFO'
+                || $id eq 'NEWMESSAGES'
+                || $id eq 'INTERNALMESSAGES'
+                || $id eq 'ALLMESSAGES'
                 || $id eq "" );
 
                @attr=split( /@@@/, $HDB{$id} );
 
                next if ( ($attr[$_CONTENT_TYPE] eq '' ||
                           $attr[$_CONTENT_TYPE] eq 'N/A' ||
-                          $attr[$_CONTENT_TYPE] =~ /^text/i) 
+                          $attr[$_CONTENT_TYPE] =~ /^text/i)
                        && $attr[$_SIZE]<4096 );
 
                next if ($attr[$_STATUS] =~ /T/i);
@@ -739,7 +816,7 @@ sub releaseupgrade {
             }
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
          filelock($folderfile, LOCK_UN);
       }
       writehistory("release upgrade - $folderdir/* by 20020108.02");
@@ -753,18 +830,17 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
 
-         next if ( ! -f "$headerdb$config{'dbm_ext'}");
-
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
          dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
 
          @messageids=keys %HDB;
          foreach my $id (@messageids) {
-            next if ( $id eq 'METAINFO' 
-             || $id eq 'NEWMESSAGES' 
-             || $id eq 'INTERNALMESSAGES' 
-             || $id eq 'ALLMESSAGES' 
+            next if ( $id eq 'METAINFO'
+             || $id eq 'NEWMESSAGES'
+             || $id eq 'INTERNALMESSAGES'
+             || $id eq 'ALLMESSAGES'
              || $id eq "" );
 
             @attr=split( /@@@/, $HDB{$id} );
@@ -775,12 +851,12 @@ sub releaseupgrade {
             } elsif ($d[2]<=1900) {
                $d[2]+=1900;
             }
-            $attr[$_DATE]=sprintf("%4d%02d%02d%02d%02d%02d", 
+            $attr[$_DATE]=sprintf("%4d%02d%02d%02d%02d%02d",
 					$d[2],$d[0],$d[1], $d[3],$d[4],$d[5]);
             $HDB{$id}=join('@@@', @attr);
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
 
          my $cachefile="$headerdb.cache";
          ($cachefile =~ /^(.+)$/) && ($cachefile = $1);  # untaint ...
@@ -790,7 +866,64 @@ sub releaseupgrade {
       writelog("release upgrade - $folderdir/.*$config{'dbm_ext'} by 20020108.02");
    }
 
-   if ( $user_releasedate lt "20020329" ) {
+   if ( $user_releasedate lt "20020601" ) {
+      my $timeoffset=gettimeoffset();
+      $timeoffset=~s/\+/-/ || $timeoffset=~s/\-/+/;	# switch +/-
+      my (@validfolders, $folderusage);
+      getfolders(\@validfolders, \$folderusage);
+
+      foreach my $foldername (@validfolders) {
+         my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
+         my (%HDB, @messageids, @attr);
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
+
+         filelock($folderfile, LOCK_SH);
+         open (FOLDER, $folderfile);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
+         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+
+         @messageids=keys %HDB;
+         foreach my $id (@messageids) {
+            next if ( $id eq 'METAINFO'
+                   || $id eq 'NEWMESSAGES'
+                   || $id eq 'INTERNALMESSAGES'
+                   || $id eq 'ALLMESSAGES'
+                   || $id eq "" );
+            my ($buff, $delimiter, $datefield, $dateserial);
+            @attr=split( /@@@/, $HDB{$id} );
+            seek(FOLDER, $attr[$_OFFSET], 0);
+            if (length($attr[$_FROM].$attr[$_TO].$attr[$_SUBJECT].$attr[$_CONTENT_TYPE].$attr[$_REFERENCES])>384) {
+                read(FOLDER, $buff, 2048);
+            } else {
+                read(FOLDER, $buff, 1024);
+            }
+            if ( $buff =~ /^From (.+?)\n/ims) {
+               $delimiter=$1;
+               if ( $buff =~ /\nDate: (.+?)\n/ims ) {
+                  $datefield=$1;
+               }
+               my $dateserial=datefield2dateserial($datefield);
+               my $deliserial=delimiter2dateserial($delimiter, $config{'deliver_use_GMT'});
+               if ($dateserial eq "" ||
+                   ($deliserial ne "" && dateserial2daydiff($dateserial)-dateserial2daydiff($deliserial)>1) ) {
+                  $dateserial=$deliserial; # use receiving time if sending time is newer than receiving time
+               }
+               $dateserial=gmtime2dateserial() if ($dateserial eq "");
+               $attr[$_DATE]=$dateserial;
+            } else {
+               $attr[$_DATE]=add_dateserial_timeoffset($attr[$_DATE], $timeoffset);
+            }
+            $HDB{$id}=join('@@@', @attr);
+         }
+         dbmclose(%HDB);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+         filelock($folderfile, LOCK_UN);
+      }
+      writehistory("release upgrade - $folderdir/* by 20020601");
+      writelog("release upgrade - $folderdir/* by 20020601");
+   }
+
+   if ( $user_releasedate lt "20020728" ) {
       $rc_upgrade=1;	# .openwebmailrc upgrade will be requested
    }
 
