@@ -1,10 +1,18 @@
-# 
+#
 # auth_unix.pl - authenticate user with unix password
 #
-# $unix_passwdfile : the location of the file containing all usernames 
-#                    and their corresponding encrypted passwords. 
+# 2002/07/19 tung@turtle.ee.ncku.edu.tw
+#
+
+#
+# $unix_passwdfile_plaintext : the plaintext file containing all usernames 
+#                              and related uid, gid, homedir, shell info.
+#                              The deault is /etc/passwd on most unix systems.
+#
+# $unix_passwdfile_encrypted : the file containing all usernames and
+#                              their corresponding encrypted passwords.
 # ------------------------------   --------------------------------
-# platform                         passwdfile
+# platform                         passwdfile encrypted
 # ------------------------------   --------------------------------
 # Shadow Passwd (linux/solaris)    /etc/shadow
 # FreeBSD                          /etc/master.passwd
@@ -14,7 +22,7 @@
 # else...                          /etc/passwd
 # ------------------------------   --------------------------------
 #
-# $unix_passwdmkdb : The command executed after any password modification 
+# $unix_passwdmkdb : The command executed after any password modification
 #                    to update the changes of passwdfile to passwd database.
 # ------------------------------   --------------------------------
 # platform                         passwd mkdb command
@@ -24,10 +32,9 @@
 # else...                          none
 # ------------------------------   --------------------------------
 #
-# 2001/12/20 tung@turtle.ee.ncku.edu.tw
-#
 
-my $unix_passwdfile="/etc/master.passwd";
+my $unix_passwdfile_plaintext="/etc/passwd";
+my $unix_passwdfile_encrypted="/etc/master.passwd";
 my $unix_passwdmkdb="/usr/sbin/pwd_mkdb";
 
 ################### No configuration required from here ###################
@@ -37,7 +44,13 @@ use Fcntl qw(:DEFAULT :flock);
 
 sub get_userinfo {
    my $user=$_[0];
-   my ($uid, $gid, $realname, $homedir) = (getpwnam($user))[2,3,6,7];
+   my ($uid, $gid, $realname, $homedir);
+
+   if ($unix_passwdfile_plaintext eq "/etc/passwd") {
+      ($uid, $gid, $realname, $homedir)= (getpwnam($user))[2,3,6,7];
+   } else {
+      ($uid, $gid, $realname, $homedir)= (getpwnam_file($user, $unix_passwdfile_plaintext))[2,3,6,7];
+   }
 
    # use first field only
    $realname=(split(/,/, $realname))[0];
@@ -51,16 +64,16 @@ sub get_userinfo {
 
 sub get_userlist {	# only used by checkmail.pl -a
    my @userlist=();
-   my $line;   
+   my $line;
 
    # a file should be locked only if it is local accessable
-   filelock("$unix_passwdfile", LOCK_SH) if ( -f $unix_passwdfile);
-   open(PASSWD, $unix_passwdfile);
+   filelock("$unix_passwdfile_encrypted", LOCK_SH) if ( -f $unix_passwdfile_encrypted);
+   open(PASSWD, $unix_passwdfile_encrypted);
    while (defined($line=<PASSWD>)) {
       push(@userlist, (split(/:/, $line))[0]);
    }
    close(PASSWD);
-   filelock("$unix_passwdfile", LOCK_UN) if ( -f $unix_passwdfile);
+   filelock("$unix_passwdfile_encrypted", LOCK_UN) if ( -f $unix_passwdfile_encrypted);
    return(@userlist);
 }
 
@@ -76,9 +89,9 @@ sub check_userpassword {
    return -2 unless ( $user ne "" && $password ne "");
 
    # a file should be locked only if it is local accessable
-   filelock("$unix_passwdfile", LOCK_SH) if ( -f $unix_passwdfile);
-   if ( ! open (PASSWD, "$unix_passwdfile") ) {
-      filelock("$unix_passwdfile", LOCK_UN) if ( -f $unix_passwdfile);
+   filelock("$unix_passwdfile_encrypted", LOCK_SH) if ( -f $unix_passwdfile_encrypted);
+   if ( ! open (PASSWD, "$unix_passwdfile_encrypted") ) {
+      filelock("$unix_passwdfile_encrypted", LOCK_UN) if ( -f $unix_passwdfile_encrypted);
       return -3;
    }
    while (defined($line=<PASSWD>)) {
@@ -86,11 +99,11 @@ sub check_userpassword {
       last if ($u eq $user); # We've found the user in /etc/passwd
    }
    close (PASSWD);
-   filelock("$unix_passwdfile", LOCK_UN) if ( -f $unix_passwdfile);
+   filelock("$unix_passwdfile_encrypted", LOCK_UN) if ( -f $unix_passwdfile_encrypted);
 
    if ($u eq $user && crypt($password,$p) eq $p) {
       return 0;
-   } else { 
+   } else {
       return -4;
    }
 }
@@ -111,10 +124,10 @@ sub change_userpassword {
    return -2 if (length($newpassword)<4);
 
    # a passwdfile could be modified only if it is local accessable
-   return -1 if (! -f $unix_passwdfile);
+   return -1 if (! -f $unix_passwdfile_encrypted);
 
-   filelock("$unix_passwdfile", LOCK_EX);
-   open (PASSWD, $unix_passwdfile) or return -3;
+   filelock("$unix_passwdfile_encrypted", LOCK_EX);
+   open (PASSWD, $unix_passwdfile_encrypted) or return -3;
    while (defined($line=<PASSWD>)) {
       $content .= $line;
       if ($u ne $user) {
@@ -124,7 +137,7 @@ sub change_userpassword {
    close (PASSWD);
 
    if ($u ne $user || crypt($oldpassword,$p) ne $p) {
-      filelock("$unix_passwdfile", LOCK_UN);
+      filelock("$unix_passwdfile_encrypted", LOCK_UN);
       return -4;
    }
 
@@ -143,33 +156,58 @@ sub change_userpassword {
    my $newline=join(":", $u, $encrypted, $misc);
 
    if ($content !~ s/\Q$oldline\E/$newline/) {
-      filelock("$unix_passwdfile", LOCK_UN);
+      filelock("$unix_passwdfile_encrypted", LOCK_UN);
       return -3;
    }
 
-   open(TMP, ">$unix_passwdfile.tmp.$$") || goto authsys_error;
+   open(TMP, ">$unix_passwdfile_encrypted.tmp.$$") || goto authsys_error;
    print TMP $content || goto authsys_error;
    close(TMP) || goto authsys_error;
 
    if ($unix_passwdmkdb ne "" && $unix_passwdmkdb ne "none" ) {
       # update passwd and db with pwdmkdb program
-      if ( system("$unix_passwdmkdb $unix_passwdfile.tmp.$$")!=0 ) {
+      if ( system("$unix_passwdmkdb $unix_passwdfile_encrypted.tmp.$$")!=0 ) {
          goto authsys_error;
       }
    } else {
       # automic update passwd by rename
-      my ($fmode, $fuid, $fgid) = (stat($unix_passwdfile))[2,4,5];
-      chown($fuid, $fgid, "$unix_passwdfile.tmp.$$");
-      chmod($fmode, "$unix_passwdfile.tmp.$$");
-      rename("$unix_passwdfile.tmp.$$", $unix_passwdfile) || goto authsys_error;
+      my ($fmode, $fuid, $fgid) = (stat($unix_passwdfile_encrypted))[2,4,5];
+      chown($fuid, $fgid, "$unix_passwdfile_encrypted.tmp.$$");
+      chmod($fmode, "$unix_passwdfile_encrypted.tmp.$$");
+      rename("$unix_passwdfile_encrypted.tmp.$$", $unix_passwdfile_encrypted) || goto authsys_error;
    }
-   filelock("$unix_passwdfile", LOCK_UN);
+   filelock("$unix_passwdfile_encrypted", LOCK_UN);
    return(0);
 
 authsys_error:
-   unlink("$unix_passwdfile.tmp.$$");
-   filelock("$unix_passwdfile", LOCK_UN);
+   unlink("$unix_passwdfile_encrypted.tmp.$$");
+   filelock("$unix_passwdfile_encrypted", LOCK_UN);
    return(-3);
 }
 
+
+# this routie is slower than system getpwnam() but can work with file 
+# other than /etc/passwd. ps: it always return '*' for passwd field.
+sub getpwnam_file {
+   my ($user, $passwdfile_plaintext)=@_;
+   my ($name, $passwd, $uid, $gid, $gcos, $dir, $shell);
+
+   return("", "", "", "", "", "", "", "", "") if ($user eq "" || ! -f $passwdfile_plaintext);
+
+   open(PASSWD, "$passwdfile_plaintext");
+   while(<PASSWD>) {
+      next if (/^#/);
+      chomp;
+      ($name, $passwd, $uid, $gid, $gcos, $dir, $shell)=split(/:/);
+      last if ($name eq $user);
+   }
+   close(PASSWD);
+
+   if ($name eq $user) {
+      return($name, "*", $uid, $gid, 0, "", $gcos, $dir, $shell);
+   } else {
+      return("", "", "", "", "", "", "", "", "");
+   }
+}
+
 1;
